@@ -7,6 +7,8 @@ import time
 import json
 import numpy as np
 from datetime import date, datetime
+import pandas as pd
+from typing import Iterable, Mapping
 
 SERVER_KEY = "ae_server_address"
 
@@ -29,25 +31,26 @@ def AnnotationClient(server_address,
                      api_version='latest',
                      verify=True):
     """ Factory for returning AnnotationClient
-    Parameters
-    ----------
-    server_address : str 
-        server_address to use to connect to (i.e. https://minniev1.microns-daf.com)
-    datastack_name : str
-        Name of the datastack.
-    auth_client : AuthClient or None, optional
-        Authentication client to use to connect to server. If None, do not use authentication.
-    api_version : str or int (default: latest)
-        What version of the api to use, 0: Legacy client (i.e www.dynamicannotationframework.com) 
-        2: new api version, (i.e. minniev1.microns-daf.com)
-        'latest': default to the most recent (current 2)
-    verify : str (default : True)
-        whether to verify https
+    
+        Parameters
+        ----------
+        server_address : str 
+            server_address to use to connect to (i.e. https://minniev1.microns-daf.com)
+        datastack_name : str
+            Name of the datastack.
+        auth_client : AuthClient or None, optional
+            Authentication client to use to connect to server. If None, do not use authentication.
+        api_version : str or int (default: latest)
+            What version of the api to use, 0: Legacy client (i.e www.dynamicannotationframework.com) 
+            2: new api version, (i.e. minniev1.microns-daf.com)
+            'latest': default to the most recent (current 2)
+        verify : str (default : True)
+            whether to verify https
 
-    Returns
-    -------
-    ClientBaseWithDatastack
-        List of datastack names for available datastacks on the annotation engine
+        Returns
+        -------
+        ClientBaseWithDatastack
+            List of datastack names for available datastacks on the annotation engine
     """
 
     if auth_client is None:
@@ -80,7 +83,7 @@ class AnnotationClientLegacy(ClientBaseWithDataset):
             List of dataset names for available datasets on the annotation engine
         """
         url = self._endpoints["datasets"].format_map(self.default_url_mapping)
-        response = self.session.get(url, verify=self.verify)
+        response = self.session.get(url)
         return handle_response(response)
 
     def get_tables(self, dataset_name=None):
@@ -102,7 +105,7 @@ class AnnotationClientLegacy(ClientBaseWithDataset):
         endpoint_mapping["dataset_name"] = dataset_name
         url = self._endpoints["table_names"].format_map(endpoint_mapping)
 
-        response = self.session.get(url, verify=self.verify)
+        response = self.session.get(url)
         return handle_response(response)
 
     def create_table(self, table_name, schema_name, dataset_name=None):
@@ -131,7 +134,7 @@ class AnnotationClientLegacy(ClientBaseWithDataset):
         url = self._endpoints["table_names"].format_map(endpoint_mapping)
         data = {"schema_name": schema_name, "table_name": table_name}
 
-        response = requests.post(url, json=data, verify=self.verify)
+        response = requests.post(url, json=data)
         return handle_response(response)
 
     def get_annotation(self, table_name, annotation_id, dataset_name=None):
@@ -161,7 +164,7 @@ class AnnotationClientLegacy(ClientBaseWithDataset):
 
         url = self._endpoints["existing_annotation"].format_map(
             endpoint_mapping)
-        response = self.session.get(url, verify=self.verifys)
+        response = self.session.get(url)
         return handle_response(response)
 
     def post_annotation(self, table_name, data, dataset_name=None):
@@ -194,7 +197,7 @@ class AnnotationClientLegacy(ClientBaseWithDataset):
         url = self._endpoints["new_annotation"].format_map(endpoint_mapping)
 
         response = self.session.post(url, data=json.dumps(data, cls=AEEncoder),
-                                     headers={'Content-Type': 'application/json'}, verify=self.verify)
+                                     headers={'Content-Type': 'application/json'})
         return handle_response(response)
 
 
@@ -231,7 +234,7 @@ class AnnotationClientV2(ClientBase):
         endpoint_mapping["aligned_volume_name"] = aligned_volume_name
         url = self._endpoints["tables"].format_map(endpoint_mapping)
         print(url)
-        response = self.session.get(url, verify=self.verify)
+        response = self.session.get(url)
         return handle_response(response)
 
     def get_annotation_count(self, table_name: str, aligned_volume_name=None):
@@ -259,7 +262,7 @@ class AnnotationClientV2(ClientBase):
 
         url = self._endpoints["table_count"].format_map(endpoint_mapping)
 
-        response = self.session.get(url, verify=self.verify)
+        response = self.session.get(url)
         return handle_response(response)
 
     def get_table_metadata(self, table_name: str, aligned_volume_name=None):
@@ -287,7 +290,7 @@ class AnnotationClientV2(ClientBase):
 
         url = self._endpoints["table_info"].format_map(endpoint_mapping)
 
-        response = self.session.get(url, verify=self.verify)
+        response = self.session.get(url)
         return handle_response(response)
 
     def delete_table(self, table_name: str, aligned_volume_name=None):
@@ -316,7 +319,7 @@ class AnnotationClientV2(ClientBase):
 
         url = self._endpoints["table_info"].format_map(endpoint_mapping)
 
-        response = self.session.delete(url, verify=self.verify)
+        response = self.session.delete(url)
         return handle_response(response)
 
     def create_table(self, table_name, schema_name,
@@ -379,7 +382,7 @@ class AnnotationClientV2(ClientBase):
                 "table_name": table_name,
                 "metadata": metadata}
 
-        response = self.session.post(url, json=data, verify=self.verify)
+        response = self.session.post(url, json=data)
         return handle_response(response)
 
     def get_annotation(self, table_name, annotation_ids, aligned_volume_name=None):
@@ -414,7 +417,7 @@ class AnnotationClientV2(ClientBase):
         params = {
             'annotation_ids': ",".join([str(a) for a in annotation_ids])
         }
-        response = self.session.get(url, params=params, verify=self.verify)
+        response = self.session.get(url, params=params)
         return handle_response(response)
 
     def post_annotation(self, table_name, data, aligned_volume_name=None):
@@ -455,9 +458,72 @@ class AnnotationClientV2(ClientBase):
         }
 
         response = self.session.post(url, data=json.dumps(data, cls=AEEncoder),
-                                     headers={'Content-Type': 'application/json'}, verify=self.verify)
+                                     headers={'Content-Type': 'application/json'})
         return handle_response(response)
 
+    @staticmethod
+    def process_position_columns(df: pd.DataFrame,
+                                 position_columns: (Iterable[str] or Mapping[str, str] or None)):
+        """process a dataframe into a list of dictionaries, nesting thing
+
+        Args:
+            df (pd.DataFrame): dataframe to process
+            position_columns (Iterable[str] or Mapping[str, str] or None): see post_annotation_df
+        Returns:
+            json list of annotations ready for posting
+
+        """
+        if position_columns is None:
+            position_columns = [c for c in df.columns if c.endswith('_position')]
+        if isinstance(position_columns, (list, np.ndarray, pd.Index)):
+            position_columns = {c:c.rsplit('_',1)[0] for c in position_columns}
+        if type(position_columns)!=dict:
+            raise ValueError('position_columns must be a list, dict or None')
+
+        data=df.to_dict(orient='records')
+        for d in data:
+            for k,v in position_columns.items():
+                pos = d.pop(k)
+                d[v]={'position':pos}
+        return data
+
+    def post_annotation_df(self, table_name: str, df: pd.DataFrame,
+                           position_columns: (Iterable[str] or Mapping[str, str] or None),
+                           aligned_volume_name=None):
+        """ Post one or more new annotations to a table in the AnnotationEngine
+
+        Parameters
+        ----------
+        table_name : str
+            Name of the table where annotations will be added
+        df : pd.DataFrame
+            A pandas dataframe containing the annotations. Columns should be fields in schema,
+            position columns need to be called out in position_columns argument.
+        position_columns: dict or (list or np.array or pd.Index) or None
+            if None, will look for all columns with 'X_position' in the name and assume they go
+            in fields called "X". 
+            if Iterable assumes each column given ends in _position.
+            (i.e. ['pt_position'] if 'pt' is the name of the position field in schema) 
+            if Mapping, keys are names of columns in dataframe, values are the names of the fields
+            (i.e. {'pt_column': 'pt'} would be correct if you had one column named 'pt_column'
+            which needed to go into a schema with a position column called 'pt')
+            
+        aligned_volume_name : str or None, optional
+            Name of the aligned_volume. If None, uses the one specified in the client.
+
+        Returns
+        -------
+        json
+            Response JSON
+
+        """
+        if aligned_volume_name is None:
+            aligned_volume_name = self.aligned_volume_name
+        
+        data = self.process_position_columns(df, position_columns)
+
+        return self.post_annotation(table_name, data, aligned_volume_name=aligned_volume_name)
+
     def update_annotation(self, table_name, data, aligned_volume_name=None):
         """Update one or more new annotations to a table in the AnnotationEngine
         Note update is implemented by deleting the old annotation
@@ -498,9 +564,46 @@ class AnnotationClientV2(ClientBase):
             "annotations": data
         }
 
-        response = self.session.put(url, json=data, verify=self.verify)
+        response = self.session.put(url, json=data)
         return handle_response(response)
 
+    def update_annotation_df(self, table_name: str, df: pd.DataFrame,
+                           position_columns: (Iterable[str] or Mapping[str, str] or None),
+                           aligned_volume_name=None):
+        """ Update one or more  annotations to a table in the AnnotationEngine using a dataframe as format
+
+        Parameters
+        ----------
+        table_name : str
+            Name of the table where annotations will be added
+        df : pd.DataFrame
+            A pandas dataframe containing the annotations. Columns should be fields in schema,
+            position columns need to be called out in position_columns argument.
+        position_columns: dict or (list or np.array or pd.Index) or None
+            if None, will look for all columns with 'X_position' in the name and assume they go
+            in fields called "X". 
+            if Iterable assumes each column given ends in _position.
+            (i.e. ['pt_position'] if 'pt' is the name of the position field in schema) 
+            if Mapping, keys are names of columns in dataframe, values are the names of the fields
+            (i.e. {'pt_column': 'pt'} would be correct if you had one column named 'pt_column'
+            which needed to go into a schema with a position column called 'pt')
+            
+        aligned_volume_name : str or None, optional
+            Name of the aligned_volume. If None, uses the one specified in the client.
+
+        Returns
+        -------
+        json
+            Response JSON
+
+        """
+        if aligned_volume_name is None:
+            aligned_volume_name = self.aligned_volume_name
+        
+        data = self.process_position_columns(df, position_columns)
+
+        return self.update_annotation(table_name, data, aligned_volume_name=aligned_volume_name)
+        
     def delete_annotation(self, table_name, annotation_ids, aligned_volume_name=None):
         """Update one or more new annotations to a table in the AnnotationEngine
         Note update is implemented by deleting the old annotation
@@ -540,7 +643,7 @@ class AnnotationClientV2(ClientBase):
         }
 
         response = self.session.delete(url, data=json.dumps(data, cls=AEEncoder),
-                                       headers={'Content-Type': 'application/json'}, verify=self.verify)
+                                       headers={'Content-Type': 'application/json'})
         return handle_response(response)
 
 
